<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Teknisa\CRM\Service\GeralService;
use Teknisa\CRM\Controller\ReplicacaoParceiroController as Replicacao;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\CRM\Util\ConsultaInscricaoRF;
use Teknisa\CRM\Controller\crm20000CasoController as CasoController;
use Teknisa\Model\Util\NovoCodigo;

class crm00004000LeadController extends TeknisaCrud {
    protected $dataSourceName = 'crm_candidato_principal';
    protected $primaryKey = 'NRCANDIDATO';
    protected $parameterBagColumns = array('P_NRPARCNEGOCIO', 'P_NRORG', 'IDTPOPERADOR', 'NRREVENDA', 'IDATIVO', 'P_CRM_NRORGTRAB');
    //protected $CDCONTADOR = 'CANDIDATO9999';

    protected $entityManager;
    protected $connection;
    protected $procedureFundirParcnegocio;
    protected $newCode;
    protected $environment;
    protected $leadService;
    protected $agrupamentoParceiroService;
    protected $agrupamentoParceiroController;

    public function __construct ($crudService, $parameterBag, $entityManager, StoredProcedure $procedureFundirParcnegocio, $newCode, $environment, $leadService, Replicacao $replicacao, $controlerNovaTarefa, $agrupamentoParceiroService, $agrupamentoParceiroController)
    {
        
        $this->entityManager                  = $entityManager;
        $this->connection                     = $entityManager->getConnection();
        $this->procedureFundirParcnegocio     = $procedureFundirParcnegocio;
        $this->newCode                        = $newCode;   
        $this->environment                    = $environment;   
        $this->leadService                    = $leadService;
        $this->agrupamentoParceiroController  = $agrupamentoParceiroController;
        $this->agrupamentoParceiroService     = $agrupamentoParceiroService;
        $this->replicacao                     = $replicacao;
        $this->controlerNovaTarefa            = $controlerNovaTarefa;
        
        parent::__construct($crudService, $parameterBag);

    }
    
    public function validaLead(DTO\Request $row, DTO\Response $response){
        try{
            $row = $row->getRow();
            
            $row['PKVALUE']   = isset($row['NRPARCNEGOCIO']) ? $row['NRPARCNEGOCIO'] : null;
            
            $return = $this->leadService->validaLead($row);
            $leads = array();
            if(empty($return)){
                $leads = false;
                $parcExists = false;
            }else if(is_array($return)){
                if($row['__is_new']) {
                    foreach($return as $lead) {
                        if(!in_array($lead, $leads)){
                            array_push($leads, $lead);
                        }
                    }
                    $parcExists = true;
                } else {
                    if(count($return) > 1) {
                        foreach($return as $lead) {
                            if(!in_array($lead, $leads)){
                                array_push($leads, $lead);
                            }
                        }
                        $parcExists = true;
                    }
                }
            }else{
                if(isset($row['IDVALIDACNPJ']) && $row['IDVALIDACNPJ'] == 'S') {
                    $leads = true;
                } else {
                    $leads = false;
                }
                $parcExists = true;
            }
            
            if($leads === false) {
                $parcExists = true;
                $leads = $this->leadService->searchForParc($row);
                if(!empty($leads)) {
                    $orgs = $this->leadService->parcRevalidation($row['NRORG'], $leads['NRPARCNEGOCIO']);
                    if(!empty($orgs[0])) {
                        foreach($orgs[0] as $org) {
                            if($org == $row['CRM_NRORGTRAB'] && $row['__is_new'] == true) {
                                $leads = true;
                            }
                        }
                    }
                }
            }
            
            $response->addDataSet(new DataSet("DADOS", array("LEADS" => $leads, "PARCEXISTS" => $parcExists)));
                
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaParametrizacaoProspect(DTO\Request $row, DTO\Response $response){
        try{
            $row = $row->getRow();
            
            $arrayLimites = $this->leadService->verificaParametrizacaoProspect($row);
            
            $response->addDataSet(new DataSet("DADOS", $arrayLimites));
                
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaOperadorRevenda(DTO\Request $row, DTO\Response $response){
        try{
            $row = $row->getRow();
            
            if(!empty($row['IDTPOPERADOR']) && $row['IDTPOPERADOR'] === 'R') {
                $arrayLimites = $this->leadService->validaOperadorRevenda($row);
            } else {
                $arrayLimites = array("EXCEDEULIMITE" => false);
            }
            
            $response->addDataSet(new DataSet("DADOS", $arrayLimites));
                
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $this->connection->beginTransaction();
            
            foreach($request->getDataSet()->getRows() as &$row) {
                $criaNovoContato = false;
                $fundeParc = false;
                $row['IDTPIJURCAN'] = !empty($row['IDTPIJURCAN']) ? $row['IDTPIJURCAN'] : 'I';
                $row['CDOPERRESPONSAVEL'] = $this->leadService->getCdOperador($row['NRPARCPROPRI']);
                $row['NRPARCRESP'] = $row['NRPARCPROPRI'];
                $row['CDPAIS'] = empty($row['CDPAIS']) ? '' : $row['CDPAIS'];
                $row['SGESTADO'] = empty($row['SGESTADO']) ? '' : $row['SGESTADO'];
                $row['CDMUNICIPIO'] = empty($row['CDMUNICIPIO']) ? '' : $row['CDMUNICIPIO'];
                $row['CDBAIRRO'] = empty($row['CDBAIRRO']) ? '' : $row['CDBAIRRO'];
                if ($row['__is_new']) {
                    $row["CDFILIAL"] = $this->environment->getCdFilial();
                    $row["CDTIPOPARCEIRO"] = "CANDIDATO";
                    
                    $parcCriado = $this->leadService->verificaParceiro($row);
                    if(empty($parcCriado)) {
                        $row['NRCANDIDATO'] = $this->newCode->getCodeByTableName("CANDIDATO9999", $row["NRORG"]);
                        $row['NRPARCNEGOCIO'] = $this->newCode->getCodeByTableName("PARCNEGOCIO", $row["NRORG"]);
                        $row['NRSEQCONTATO'] = $this->newCode->getCodeByTableName("NRSEQCONTATO", $row["NRORG"]);
                        $row['NRCONTATO'] = $row['NRSEQCONTATO'];
                        $row['__is_new'] = true;
                        $criaNovoContato = true;
                        if(empty($row['NRPARCNEGOCIO'])) {
                            $row['NRPARCNEGOCIO'] = $parcCriado['NRPARCNEGOCIO'];
                        }
                    } else {
                        if(empty($row['NRCANDIDATO'])) {
                            $row['NRCANDIDATO'] = $this->newCode->getCodeByTableName("CANDIDATO9999", $row["NRORG"]);
                            $row['__is_new'] = true;
                        } else {
                            $row['__is_new'] = false;
                        }
                        if(empty($row['NRSEQCONTATO'])) {
                            $row['NRSEQCONTATO'] = $this->newCode->getCodeByTableName("NRSEQCONTATO", $row["NRORG"]);
                            $row['NRCONTATO'] = $row['NRSEQCONTATO'];
                            $criaNovoContato = true;
                        }

                    }
                     
                    //Insere o Parceiro de Negócio
                    if(!empty($parcCriado) && !empty($parcCriado['NRPARCNEGOCIO']) && empty($row['NRPARCNEGOCIO'])) {
                        $row['NRPARCNEGOCIO'] = $parcCriado['NRPARCNEGOCIO'];
                    }
                    $this->leadService->insertParcnegocio($row, !empty($parcCriado));
                    
                    //Esta função precisa estar aqui pois o contato só é adicionado após a adição do Lead/Parceiro
                    if($criaNovoContato) {
                        self::newContato($row);
                    }
                    if(empty($parcCriado)) { 
                        self::createPartnership($row);
                    } else if(isset($parcCriado['LEADEXISTS']) && !$parcCriado['LEADEXISTS']) {
                        self::createPartnershipNotExistent($row, 'CANDIDATO');
                    }
                    if($row['NRMODORECEBINFO']){
                        self::setModoRecebInfo($row);
                    }
                    if(!empty($row["NMINTERLOCUTOR"])){
                        $row['NRSEQINTERLOCUTOR'] = NovoCodigo::geraCodigo($row['NRORG'], "CRM_INTERLOCUTOR", 1, 10, false, true);
                        $this->leadService->newInterlocutor($row);
                    }
                    
                }else{
                    $parcCriado = $this->leadService->verificaParceiro($row);
                    if(!empty($parcCriado) && !empty($row['PARCORIGINAL'] && $row['PARCORIGINAL'] !== $row['NRPARCNEGOCIO'])) {
                        $row["CDTIPOPARCEIRO"] = "CANDIDATO";
                        $this->leadService->fundeParceiro($row);
                        $this->leadService->desativarStatusParc($row);
                        $this->leadService->insertStatusParc($row);
                        
                    }
                    if(self::verificaModoRecebInfo($row)){
                        self::updateModoRecebInfo($row);
                    }else if($row['NRMODORECEBINFO']){
                        self::setModoRecebInfo($row);
                    }
                }
                
                $this->saveComunicaparc($row, $response);
                $this->saveDtnascifundparc($row);
                self::saveEnderecoparc($row);
            }
            //Insere o Lead a partir de um Insert direto no banco
            $this->dataSourceName = 'leadSave';
            parent::save($request, $response);
            $this->updateResponsavel($row);
            
            $parentError = $response->getError();
            
            if (!empty($parentError)) {
                throw new \Exception($parentError->getMessage(), $parentError->getErrorCode());
            }
            
            $this->leadService->alteraAtivoStatusParc($row);
            
            $this->dataSourceName = 'crm_candidato_principal';
            
            //Faz replicação
            $replicacao = $this->replicacao->replica($row['NRPARCNEGOCIO'], 'CANDIDATO', 'CANDIDATO');
            $response->addDataSet(new DataSet("REPLICACAO", array($replicacao)));
            $response->addDataSet(new DataSet("DADOS", array("NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"], "NRCONTATO" => $row["NRSEQCONTATO"], "NRSEQCONTATO" => $row["NRSEQCONTATO"], "NRCANDIDATO" => $row["NRCANDIDATO"])));
            
            $this->connection->commit();
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
            $this->connection->rollBack();
        }
    }
    
    public function delete(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        self::setLeadIdAtivo($request);
     
    }
    
    public function setLeadIdAtivo($request){
        
        $row = $request->getRow();
        
        $updateParams = array(
            "NRPARCNEGOCIO"   => $row["NRPARCNEGOCIO"],
            "IDATIVO"         => $row["IDATIVO"],
            "CRM_NRORGTRAB"   => $row["CRM_NRORGTRAB"],
            "NRORG"           => $row["NRORG"]
        );
        
        $this->leadService->callToSetLeadIdAtivo($updateParams);
      
    }
    
    public function setQualification(Row $row, Response $response) {
        $row = $row->getRow();

        $update = "UPDATE CRM_STATUSPARC
                   SET IDSTATUS      = :IDSTATUSCAN,
                       DTULTATU      =  SYSDATE,
                       CDOPERULTATU  = :CDOPERULTATU
                   WHERE 
                     NRPARCNEGOCIO = :NRPARCNEGOCIO 
                     AND NRORG = :NRORG
                     AND CRM_NRORGTRAB = :CRM_NRORGTRAB";

        $params = array (
            "IDSTATUSCAN"    => $row["IDSTATUSCAN"],
            "CDOPERULTATU"   => $row["CDOPERULTATU"],
            "NRPARCNEGOCIO"  => $row["NRPARCNEGOCIO"],
            "NRORG"          => $row["NRORG"],
            "CRM_NRORGTRAB"  => $row["CRM_NRORGTRAB"]
        );
        
        try {
            $std = $this->entityManager->getConnection()->prepare($update);
            $std->execute($params);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        };
    }
    
    public function verificaModoRecebInfo($row){
      $verificamodorecebinfo = "SELECT NRPARCNEGOCIO FROM GER_PARCNEGOCIOINFO where NRPARCNEGOCIO  =  :NRPARCNEGOCIO and NRORG = :NRORG";
       
      $params = array
        (
            "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],
            "NRORG"         => $this->environment->getNrOrg()
       
        );
        
        $std = $this->connection->prepare($verificamodorecebinfo);
        $std->execute($params);
        $retornoModoRecebInfo = $std->fetchAll();
        
        if (empty($retornoModoRecebInfo)) {
            return false;    
        }
        
        return true;
    }    
        
    
    public function updateModoRecebInfo($row){
        $updatemodorecebeinfo = "UPDATE GER_PARCNEGOCIOINFO
                                 SET NRMODORECEBINFO = :NRMODORECEBINFO
                                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO AND
                                 NRORG = :NRORG";
        $params = array (
            "NRMODORECEBINFO" => empty($row['NRMODORECEBINFO']) ? null : $row['NRMODORECEBINFO'],
            "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],
            "NRORG" => $row['NRORG']
        );
        
        $std = $this->connection->prepare($updatemodorecebeinfo);
        $std->execute($params);
    }
    
    public function deactivateParc(DTO\Request $row, DTO\Response $response) {
        $row = $row->getRow();
        
        $query = "UPDATE 
                    CRM_STATUSPARC 
                  SET 
                    IDATIVO = :IDATIVO
                  WHERE 
                    NRPARCNEGOCIO = :NRPARCNEGOCIO 
                    AND NRORG = :NRORG
                    AND CRM_NRORGTRAB = :CRM_NRORGTRAB";
                    
        $params = array (
            "IDATIVO"       => 'N',
            "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],
            "NRORG"         => $row['NRORG'],
            "CRM_NRORGTRAB" => $row['CRM_NRORGTRAB']
        );
        
        $std = $this->connection->prepare($query);
        $std->execute($params);
    }
     
    public function updateLead($row) {
        try {
            $updateLead   = "UPDATE CRM_CANDIDATO
                             SET NRSEQPERFILRESPONSAVEL = :NRSEQPERFILRESPONSAVEL,
                                 IDPARCFUNDIDO = :IDPARCFUNDIDO,
                                 NMCANDIDATO = :NMCANDIDATO,
                                 IDTPIJURCAN = :IDTPIJURCAN,
                                 NRINSJURCAN = :NRINSJURCAN,
                                 CDSEGMER = :CDSEGMER,
                                 DESCTITULOCAN = :DESCTITULOCAN,
                                 NMFANTCAN = :NMFANTCAN,
                                 DSEMAILCAN = :DSEMAILCAN,
                                 DSTELEFONECAN = :DSTELEFONECAN,
                                 NRFUNCCAN = :NRFUNCCAN,
                                 IDSTATUSCAN = :IDSTATUSCAN,
                                 CDMODODIVULG = :CDMODODIVULG,
                                 NRPARCNEGOCIO = :NRPARCNEGOCIO,
                                 NRPARCPROPRI = :NRPARCPROPRI,
                                 NRCONTATO = :NRCONTATO
                             WHERE NRCANDIDATO = :NRCANDIDATO AND
                             NRORG = :NRORG";
            
            $params = array (
                "NRSEQPERFILRESPONSAVEL" => $row['NRSEQPERFILRESPONSAVEL'],
                "IDPARCFUNDIDO"          => $row['IDPARCFUNDIDO'],
                "NMCANDIDATO"            => $row['NMCANDIDATO'],
                "IDTPIJURCAN"            => $row['IDTPIJURCAN'],
                "NRINSJURCAN"            => $row['NRINSJURCAN'],
                "CDSEGMER"               => $row['CDSEGMER'],
                "DESCTITULOCAN"          => $row['DESCTITULOCAN'],
                "NMFANTCAN"              => $row['NMFANTCAN'],
                "DSEMAILCAN"             => $row['DSEMAILCAN'],
                "DSTELEFONECAN"          => $row['DSTELEFONECAN'],
                "NRFUNCCAN"              => $row['NRFUNCCAN'],
                "IDSTATUSCAN"            => $row['IDSTATUSCAN'],
                "CDMODODIVULG"           => $row['CDMODODIVULG'],
                "NRPARCNEGOCIO"          => $row['NRPARCNEGOCIO'],
                "NRPARCPROPRI"           => $row['NRPARCPROPRI'],
                "NRCONTATO"              => $row['NRCONTATO'],
                "NRCANDIDATO"            => $row['NRCANDIDATO'],
                "NRORG"                  => $row['NRORG']
            );
            
            $std = $this->connection->prepare($updateLead);
            $std->execute($params);
            
            $updateStatus = "UPDATE 
                               CRM_STATUSPARC 
                             SET 
                               IDATIVO = :IDATIVO 
                             WHERE 
                               NRPARCNEGOCIO = :NRPARCNEGOCIO 
                               AND NRORG = :NRORG 
                               AND CRM_NRORGTRAB = :CRM_NRORGTRAB";
                               
            $params2 = array(
                "IDATIVO"       => $row['IDATIVO'],
                "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],
                "NRORG"         => $row['NRORG'],
                "CRM_NRORGTRAB" => $row['CRM_NRORGTRAB']
            );
            
            $std = $this->connection->prepare($updateStatus);
            $std->execute($params2);
        } catch (\Exception $e) {   
            throw $e;
        }
    }
    
     
    public function setModoRecebInfo($row){
       
        $insertModo = "INSERT INTO GER_PARCNEGOCIOINFO (NRPARCNEGOCIO, NRORG, NRMODORECEBINFO)
         VALUES (:NRPARCNEGOCIO,
                 :NRORG,
                 :NRMODORECEBINFO)";
                
        $params = array
        (
            "NRPARCNEGOCIO"         => (int)$row["NRPARCNEGOCIO"],
            "NRORG"                 => $row["NRORG"],
            "NRMODORECEBINFO"       => $row["NRMODORECEBINFO"] 
        );

        $std = $this->entityManager->getConnection()->prepare($insertModo);
        $std->execute($params);
    }
    
    public function setParceiroNegocio($row)
    {
        $insert = "INSERT INTO PARCNEGOCIO (NRPARCNEGOCIO, NRORG, NMPRINCIPALPARC, NMSECUNDARIPARC, NRINSCRICAOPARC, IDATIVO, DTINCLUSAO,
                         NRORGINCLUSAO, CDOPERINCLUSAO, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO,IDPESSOAFISICA)
         VALUES (:NRPARCNEGOCIO,
                 :NRORG,
                 :NMPRINCIPALPARC,
                 :NMSECUNDARIPARC,
                 :NRINSCRICAOPARC,
                 :IDATIVO,
                  SYSDATE,
                 :NRORGINCLUSAO,
                 :CDOPERINCLUSAO,
                 :CDTIPOPARCPRINCIPAL,
                 :CDTIPOINSCRICAO,
                 :IDPESSOAFISICA)";

        $params = array
        (
            "IDATIVO"               => $row["IDATIVO"],
            "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
            "NRORG"                 => $row["NRORG"],
            "NMPRINCIPALPARC"       => $row["NMCANDIDATO"],
            "NMSECUNDARIPARC"       => $row["NMFANTCAN"],
            "NRINSCRICAOPARC"       => $row["NRINSJURCAN"],
            "NRORGINCLUSAO"         => $row["NRORG"],
            "CDOPERINCLUSAO"        => $row["CDOPEREVOL"],
            "CDTIPOPARCPRINCIPAL"   => "CANDIDATO",
            "CDTIPOINSCRICAO"       => $row["TIPOINSC"],
            "IDPESSOAFISICA"        => $row["ISCPF"],
        );

        try
        {
            $std = $this->entityManager->getConnection()->prepare($insert);
            $std->execute($params);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
    }

    public function newContato($row) {
        
        $filiOper = $this->leadService->verificaFilialOper($row["CDOPEREVOL"], $row["NRORG"]);
        if(is_array($filiOper)) {
            if(array_key_exists("CDFILIAL", $filiOper)) {
                $row['CDFILIAL'] = $filiOper["CDFILIAL"];
            } else {
                $row['CDFILIAL'] = null;
            }
        } else {
            $row['CDFILIAL'] = null;
        }

        $query = "INSERT INTO CONTATO (CDPROSPECT, CDCLIENTE, CDFORNECED, DSREFECONT, IDSTATUSCONT, IDTPCONTATO, CDOPERCONT, NRORG, NRPARCNEGOCIO, IDATIVO, CDFILIAL, NRCONTATO, NRSEQCONTATO, DTHRCONTATO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
        SELECT PROS.CDPROSPECT, CLIE.CDCLIENTE, FORN.CDFORNECED, :DSREFECONT, :IDSTATUSCONT, :IDTPCONTATO, :CDOPERCONT, :NRORG, :NRPARCNEGOCIO, 'S' AS IDATIVO, :CDFILIAL, :NRCONTATO, :NRSEQCONTATO, SYSDATE AS DTHRCONTATO, SYSDATE AS DTINCLUSAO, :NRORG AS NRORGINCLUSAO, :CDOPERCONT
        FROM PARCNEGOCIO PARC LEFT JOIN PROSPECT PROS
        ON PARC.NRPARCNEGOCIO = PROS.NRPARCNEGOCIO AND PARC.IDATIVO = PROS.IDATIVO AND PARC.NRORG = PROS.NRORG
        LEFT JOIN CLIENTE CLIE
        ON PARC.NRPARCNEGOCIO = CLIE.NRPARCNEGOCIO AND PARC.IDATIVO = CLIE.IDATIVO AND PARC.NRORG = CLIE.NRORG
        LEFT JOIN FORNECEDOR FORN
        ON PARC.NRPARCNEGOCIO = FORN.NRPARCNEGOCIO AND PARC.IDATIVO = FORN.IDATIVO AND PARC.NRORG = FORN.NRORG
        WHERE PARC.NRPARCNEGOCIO = :NRPARCNEGOCIO AND PARC.NRORG = :NRORG AND PARC.IDATIVO = 'S'";
        
        $std = $this->entityManager->getConnection()->prepare($query);
        $params = array(

            "DSREFECONT"    => substr('Referente ao Lead '.$row['NMCANDIDATO'], 0, 59),
            "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],
            "NRSEQCONTATO"  => $row['NRSEQCONTATO'],
            "CDOPERCONT"    => $row['CDOPEREVOL'],
            "NRCONTATO"     => $row['NRCONTATO'],
            "CDFILIAL"      => $row['CDFILIAL'],
            "NRORG"         => $row['NRORG'],
            "IDSTATUSCONT"  =>'A',
            "IDTPCONTATO"   =>'P'

        );
        
        $std->execute($params);
    }
    
    public static function consultaRF(Row $row, Response $response) {
        $row = $row->getRow();
        
        $nrinsjurcan = $row['NUMBERINSC'];
        $consulta = ConsultaInscricaoRF::consultaDadoRF($nrinsjurcan);
        
        $response->addDataSet(new DataSet("CONSULTA", $consulta));
        
    }
    
    public function confirmationLocation(Row $row, Response $response){
        
        $row = $row->getRow();
        $resultadoConsulta = $row['RESULTCONSUL'];
        
        $resultadoConfirmado = self::confirState($resultadoConsulta);
        $resultadoConfirmado = array_merge($resultadoConfirmado , self::confirmCity($resultadoConsulta));
        $resultadoConfirmado = array_merge($resultadoConfirmado , self::confirmNeighborhood($resultadoConsulta, $resultadoConfirmado[1]['CDMUNICIPIO']));
        
        $response->addDataSet(new DataSet("LOCALIZACAO", $resultadoConfirmado));
    }
    
    public function confirState($consultaRF){
        $estadoConsultado = $consultaRF['Enderecos']['Estado'];
        
        $queryEstado = "SELECT ESTADO.SGESTADO AS SGESTADO,
                             ESTADO.NMESTADO AS NMESTADO
                        FROM ESTADO
                       WHERE UPPER(ESTADO.SGESTADO) = UPPER(:ESTCONSULT)
                         AND ESTADO.CDPAIS = '0055'";   
                         
        $params = array
        (
            "ESTCONSULT" => $estadoConsultado
        );
        
        try
        {
            $std = $this->connection->prepare($queryEstado);
            $std->execute($params);
            $retornoEstado = $std->fetchAll();
            
        }catch (\Exception $e)
        {
            throw $e;
        }
        
        if($retornoEstado){
           return  $retornoEstado;
        }else{
            return array( 0 => array(
                
                "SGESTADO" => "", 
                "NMESTADO" => ""
                
            ));
        }
        
    }
    
    public function confirmCity($consultaRF){
        
        $sgestado   = $consultaRF['Enderecos']['Estado'];
        $codigoIbge = $consultaRF['Enderecos']['CodigoIBGE'];
        
        $queryMunicipio = "SELECT MUNICIPIO.CDMUNICIPIO AS CDMUNICIPIO,
                                 MUNICIPIO.NMMUNICIPIO AS NMMUNICIPIO
                            FROM MUNICIPIO
                           WHERE UPPER(MUNICIPIO.NMMUNICIPIO) = UPPER('".str_replace("'","''",$consultaRF['Enderecos']['Cidade'])."')
                             AND (CDMUNICIBGE = '$codigoIbge' OR '$codigoIbge' IS NULL)
                             AND MUNICIPIO.SGESTADO = '$sgestado'
                             AND MUNICIPIO.CDPAIS = '0055'
                             AND ROWNUM <= 1";
        
        try
        {
            $std = $this->connection->prepare($queryMunicipio);
            $std->execute();
            $retornoMunicipio = $std->fetchAll();
        }catch (\Exception $e)
        {
            throw $e;
        }
        
        if($retornoMunicipio){
           return  $retornoMunicipio;
        }else{
            return array(0 => array(
                
                "CDMUNICIPIO" => "", 
                "NMMUNICIPIO" => ""
                
            ));
        }
    }
    
    public function confirmNeighborhood($consultaRF, $cdmunicipio){
        
        $sgestado = $consultaRF['Enderecos']['Estado'];

        $queryBairro = "SELECT BAIRRO.CDBAIRRO AS CDBAIRRO,
                               BAIRRO.NMBAIRRO   AS NMBAIRRO
                          FROM BAIRRO
                         WHERE UPPER(BAIRRO.NMBAIRRO) = UPPER('".str_replace("'","''",$consultaRF['Enderecos']['Bairro'])."')
                           AND BAIRRO.CDMUNICIPIO = '$cdmunicipio'
                           AND BAIRRO.SGESTADO    = '$sgestado'
                           AND BAIRRO.CDPAIS      = '0055'
                           AND ROWNUM <= 1";
        
        try
        {
            $std = $this->connection->prepare($queryBairro);
            $std->execute();
            $retornoBairro = $std->fetchAll();
        }catch (\Exception $e)
        {
            throw $e;
        }
        
        if($retornoBairro && count($retornoBairro) != 0){
           return  $retornoBairro;
        }else if($retornoBairro == array(0) || !$retornoBairro){
            return array(0 => array(
                
                "CDBAIRRO" => "", 
                "NMBAIRRO" => ""
                
            ));
        }
    }
    
    public function convertToProspect(Row $row, Response $response){
        
        $row = $row -> getRow();
        
        /*Modificado em 21/11/17 por Arthur Herbert
            inserindo enderecoparc e prospect antes da endeprosp para 
            evitar erros na trigger de ENDEPROSP
        */
        $return = self::updatePartnerAndReturnAddress($row);
        $nrenderecoparc = $return['endereco'];
        $prospect = $return['prospect'];
        if(empty($prospect)) {
            $row['CDPROSPECT'] = self::insertProspectAndReturnCDprospect($row);
        } else {
            $row['CDPROSPECT'] = $prospect[0]['CDPROSPECT'];
        }
        
        if(!empty($row['CDPROSPECT'])) {
            if(empty($prospect)) {
                self::insertAddressProspect($row, $row['CDPROSPECT'], $nrenderecoparc);
                self::disableLead($row);
                self::createPartnershipNotExistent($row);
            }
            self::editLeadResponsibleOperator($row);
            
            if(isset($row['NRREVENDA'])){
                self::insertProspectToLoggedOperatorResale($row);
            }
            
            $this->saveComunicaparc($row, $response);
            $this->saveDtnascifundparc($row);
        } else {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("error", array("Já existe um prospect com o número do parceiro que está tentando ser convertido!")));
        }
        
    }
    
    public function returnInscriptionTypeProspect($inscription){
        
        switch($inscription){
            
            case 'J':
                $inscription = 'CNPJ';
                break;
            case 'F':
                $inscription = 'CPF';
                break;
            case 'I':
                $inscription = 'LIVRE';
                break;
            case 'O':
                $inscription = 'CODIGO';
                break;
            default:
                $inscription = 'LIVRE';
                break;
        }
          return $inscription;
    } 
    
    public function updatePartnerAndReturnAddress($row){
    
        $nrParcnegocio = $row['NRPARCNEGOCIO'];
        $nrOrg = $row['NRORG'];
        $cdoperador = $row['CDOPERINCLUSAO'];
        $cdoperresponsavel = $row['CDOPERRESPONSAVEL'];
        
        $cdtipoparcprincipal = 'PROSPECT';
        $Nrinsjurpros = $row['NRINSJURCAN'];
        $RazaoSocial = $row['NMCANDIDATO'];
        $nmFantpros = $row['NMFANTCAN'];
        $cdsegmer = $row['CDSEGMER'];
        $dtnascifundparc = !empty($row['DATAFUNDACAO']) ? $row['DATAFUNDACAO'] : "";
        $row['NRPARCRESP'] = $row['NRPARCRESPROSP'];
        
        $idtpijurcan = self::returnInscriptionTypeProspect($row['IDTPIJURCAN']);
        
        $prospect = false;
        
        $parcCriado = $this->leadService->verificaParceiro($row);
        
        if(!empty($parcCriado) && !empty($row['PARCORIGINAL'] && $row['PARCORIGINAL'] != $row['NRPARCNEGOCIO'])) {
            $row["CDTIPOPARCEIRO"] = "CANDIDATO";
            $this->leadService->fundeParceiro($row);
            $this->leadService->desativarStatusParc($row);
            $this->leadService->insertStatusParc($row);
        }
        if(!empty($parcCriado)) {
            $prospect = $this->leadService->verificaProspect($row['NRPARCNEGOCIO'], $row['NRORG']);
            if(empty($prospect)) {
                $this->leadService->updateStatusProsp($row);
                
                $queryUpdateParceiro = "UPDATE 
                                          PARCNEGOCIO 
                                        SET 
                                          CDTIPOPARCPRINCIPAL = '$cdtipoparcprincipal',
                                          NMPRINCIPALPARC     = :NMPRINCIPALPARC,
                                          NMSECUNDARIPARC     = :NMSECUNDARIPARC,
                                          NRINSCRICAOPARC     = '$Nrinsjurpros',
                                          CDTIPOINSCRICAO     = '$idtpijurcan',
                                          DTNASCIFUNDPARC     = '$dtnascifundparc',
                                          CDSEGMER            = '$cdsegmer',
                                          DTULTATU            =  SYSDATE,
                                          CDOPERULTATU        = '$cdoperador',
                                          CDOPERRESPONSAVEL   = '$cdoperresponsavel'
                                        WHERE 
                                          NRORG             = ".$nrOrg."
                                          AND NRPARCNEGOCIO = ".$nrParcnegocio;
                $params = array(
                  "NMPRINCIPALPARC" => $RazaoSocial,
                  "NMSECUNDARIPARC" => $nmFantpros
                );
                $std = $this->connection->prepare($queryUpdateParceiro);
                $std->execute($params);
                
            } else {
                $this->leadService->updateStatusProsp($row);
            }
        } else {
            $this->leadService->updateStatusProsp($row);
            
            $queryUpdateParceiro = "UPDATE 
                                      PARCNEGOCIO 
                                    SET 
                                      CDTIPOPARCPRINCIPAL = '$cdtipoparcprincipal',
                                      NMPRINCIPALPARC     = :NMPRINCIPALPARC,
                                      NMSECUNDARIPARC     = :NMSECUNDARIPARC,
                                      NRINSCRICAOPARC     = '$Nrinsjurpros',
                                      CDTIPOINSCRICAO     = '$idtpijurcan',
                                      DTNASCIFUNDPARC     = '$dtnascifundparc',
                                      CDSEGMER            = '$cdsegmer',
                                      DTULTATU            =  SYSDATE,
                                      CDOPERULTATU        = '$cdoperador',
                                      CDOPERRESPONSAVEL   = '$cdoperresponsavel'
                                    WHERE 
                                      NRORG             = ".$nrOrg."
                                      AND NRPARCNEGOCIO = ".$nrParcnegocio;
            $params = array(
              "NMPRINCIPALPARC" => $RazaoSocial,
              "NMSECUNDARIPARC" => $nmFantpros
            );
            $std = $this->connection->prepare($queryUpdateParceiro);
            $std->execute($params);
        }
        
        
        $cdpais           = isset($row['CDPAIS']) ? $row['CDPAIS'] : '';
        $sgestado         = isset($row['SGESTADO']) ? $row['SGESTADO'] : '';
        $cdmunicipio      = isset($row['CDMUNICIPIO']) ? $row['CDMUNICIPIO'] : '';
        $cdbairro         = isset($row['CDBAIRRO']) ? $row['CDBAIRRO'] : '';
        $nmbairroendereco = isset($row['NMBAIRRO']) ? $row['NMBAIRRO'] : '';
        $nrimovelendereco = isset($row['NRIMOVELENDERECO']) ? $row['NRIMOVELENDERECO'] : '';
        $dscompleendereco = isset($row['DSCOMPLEENDERECO']) ? $row['DSCOMPLEENDERECO'] : '';
        $dsendereco       = isset($row['DSENDERECO']) ? $row['DSENDERECO'] : '';
        $nrcependereco    = $row['NRCEPENDERECO'];

        $latitude = $row['LATITUDE'];
        $longitude = $row['LONGITUDE'];

        if (!empty($latitude) && !empty($longitude)) {
            $dsreferenciaende = 'Latitude: '.$latitude.' / Longitude: '.$longitude;
        } else {
            $dsreferenciaende = '';
        }

        $sql = "SELECT NRENDERECOPARC
                  FROM ENDERECOPARC
                 WHERE NRPARCNEGOCIO = ".$nrParcnegocio."
                   AND NRORG = ".$nrOrg."
                   AND CDTIPOENDERECO = 'PRINCIPAL'";
                   
        $std = $this->connection->prepare($sql);
        $std->execute();
        $retorno = $std->fetchAll();
        $sql = "";
        $nrenderecoparc = "";
            
        if (count($retorno) > 0) {
            $nrenderecoparc = (int)$retorno[0]["NRENDERECOPARC"];
            
            $sql = "UPDATE ENDERECOPARC
                     SET CDPAIS           = '$cdpais',
                         SGESTADO         = '$sgestado',
                         CDMUNICIPIO      = '$cdmunicipio',
                         CDBAIRRO         = '$cdbairro',
                         NMBAIRROENDERECO = '$nmbairroendereco',
                         DSCOMPLEENDERECO = '$dscompleendereco',
                         DSREFERENCIAENDE = '$dsreferenciaende',
                         NRIMOVELENDERECO = '$nrimovelendereco',
                         DSENDERECO       = '$dsendereco',
                     NRCEPENDERECO    = '$nrcependereco',
                         DTULTATU         = SYSDATE,
                         CDOPERULTATU     = '$cdoperador'
                   WHERE NRORG            = ".$nrOrg."
                     AND NRPARCNEGOCIO    = ".$nrParcnegocio."
                     AND CDTIPOENDERECO   = 'PRINCIPAL' ";
        } else if($cdpais) {   
            $nrenderecoparc = NovoCodigo::geraCodigo($nrOrg, "ENDERECOPARC", 1, 10, false, false);

            $sql = "INSERT INTO ENDERECOPARC
                  (NRENDERECOPARC, NRPARCNEGOCIO, CDTIPOENDERECO, NRORG, CDPAIS, SGESTADO, CDMUNICIPIO, CDBAIRRO, NMBAIRROENDERECO, DSCOMPLEENDERECO, DSREFERENCIAENDE, NRIMOVELENDERECO, DTULTATU, CDOPERULTATU, DSENDERECO, NRCEPENDERECO)
                  VALUES
                  (".$nrenderecoparc.",".$nrParcnegocio.",'PRINCIPAL',".$nrOrg.",'$cdpais','$sgestado','$cdmunicipio','$cdbairro','$nmbairroendereco','$dscompleendereco','$dsreferenciaende','$nrimovelendereco',SYSDATE,'$cdoperador','$dsendereco','$nrcependereco')";
        }
        
        if($sql) {
            try{
                $std = $this->connection->prepare($sql);
                $std->execute();
            } catch (\Exception $e) {
                throw $e;
            }
                
        }
        
        return array("endereco" => $nrenderecoparc, "prospect" => $prospect);
    }
    
    public function createPartnership($row){
        $nrParcnegocio = $row['NRPARCNEGOCIO'];
        $nrparceiro = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, true);
        $nrOrg = $row['NRORG'];
        $cdoperinclusao = isset($row['CDOPEREVOL']) ? $row['CDOPEREVOL'] : $this->environment->getCdOperador();
        $cdtipoparceiro = $row["CDTIPOPARCEIRO"]; 
        $params = array(
          "NRPARCERIA"     => $nrparceiro,
          "NRORG"          => $nrOrg,
          "NRPARCNEGOCIO"  => $nrParcnegocio,
          "CDTIPOPARCEIRO" => $cdtipoparceiro,
          "IDATIVO"        => 'S',
          "CDOPERINCLUSAO" => $cdoperinclusao,
          "NRORGINCLUSAO"  => $nrOrg);
          
        $queryInsertParceria = "INSERT INTO PARCERIA(NRPARCERIA, NRORG, NRPARCNEGOCIO, CDTIPOPARCEIRO, IDATIVO, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES(:NRPARCERIA, :NRORG, :NRPARCNEGOCIO, :CDTIPOPARCEIRO, :IDATIVO, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $std = $this->connection->prepare($queryInsertParceria);
        $std->execute($params);
    }
    
    public function insertProspectAndReturnCDprospect($row){
        
        $absentProspect = $this->leadService->verifyExistentProspect($row);
        
        if($absentProspect) {
            $queryInsertProspect = "INSERT INTO PROSPECT (IDTPIJURPROS,NRINSJURPROS,IDATIVO,NMRAZSOCPROS,NMFANTPROS,CDSEGMER,NRFUNCPROS,DTNASCPROS,IDPRIPUBPROS,CDMODODIVULG,IDLIGEXTPROS,NRORGINCLUSAO,DTINCLUSAO,CDOPERINCLUSAO,NRORG,CDPROSPECT,NRPARCNEGOCIO, NRPARCRESPROSP, DSHOPAPROS, DTINITRESERVAPROSP, IDQUALIFICADO) 
            VALUES (:IDTPIJURPROS,:NRINSJURPROS,:IDATIVO,:NMRAZSOCPROS,:NMFANTPROS,:CDSEGMER,:NRFUNCPROS,:DTNASCPROS,:IDPRIPUBPROS,:CDMODODIVULG,:IDLIGEXTPROS,:NRORGINCLUSAO,:DTINCLUSAO,:CDOPERINCLUSAO,:NRORG,:CDPROSPECT,:NRPARCNEGOCIO, :NRPARCRESPROSP, :DSHOPAPROS, :DTINITRESERVAPROSP, :IDQUALIFICADO)";
            
            $nmRazsocpros = $row['NMCANDIDATO'];
            $nmFantpros = $row['NMFANTCAN'];
    
            $idTpijurcan = $row['IDTPIJURCAN'];
            $nrInsjurcan = $row['NRINSJURCAN'];
            $nrParcnegocio = $row['NRPARCNEGOCIO'];
            $cdProspect = NovoCodigo::geraCodigo($row['NRORG'], "PROSPECT", 1, 21, false, false);
            
            $idAtivo = 'S';
            $idLigextpros = 'S';
            $idPripubpros = 'V';
            $idUtlalvpros = 'N';
            $nrOrg = $row['NRORG'];
            $cdsegmer = $row['CDSEGMER'];
            $numeroFunc = $row['NRFUNCCAN'];
            $dtInclusao = $row['CURRENTDATE'];
            $cdModoDivulg = $row['CDMODODIVULG'];
            $cdOperinclusao = $this->environment->getCdOperador();
            $dtnascpros = !empty($row['DATAFUNDACAO']) ? $row['DATAFUNDACAO'] : "";
            $nrparcresprosp = $row['NRPARCRESPROSP'];
            $homePage = !empty($row['DSHOPAPROS']) ? $row['DSHOPAPROS'] : "";
            $dataInicioReserva = !empty($row['DTINITRESERVAPROSP']) ? $row['DTINITRESERVAPROSP'] : "";
            $idQualificado = 'S';
            
            $params = array(
              "IDTPIJURPROS"        => $idTpijurcan, //fisico j outros
              "NRINSJURPROS"        => $nrInsjurcan, // numero ins
              "IDATIVO"             => $idAtivo, 
              "NMRAZSOCPROS"        => $nmRazsocpros,
              "NMFANTPROS"          => $nmFantpros,
              "CDSEGMER"            => $cdsegmer,
              "NRFUNCPROS"          => $numeroFunc,   // NR FUNCIONARIOAS
              "DTNASCPROS"          => $dtnascpros,   // DATA NASCIMENTO DO PROSPECT
              "IDPRIPUBPROS"        => $idPripubpros, // 
              "CDMODODIVULG"        => $cdModoDivulg, // como hegou até nós
              "IDLIGEXTPROS"        => $idLigextpros, 
              "NRORGINCLUSAO"       => $nrOrg,      
              "DTINCLUSAO"          => $dtInclusao,
              "CDOPERINCLUSAO"      => $cdOperinclusao,
              "NRORG"               => $nrOrg,
              "CDPROSPECT"          => $cdProspect,
              "NRPARCNEGOCIO"       => $nrParcnegocio,
              "NRPARCRESPROSP"      => $nrparcresprosp,
              "DSHOPAPROS"          => $homePage,
              "DTINITRESERVAPROSP"  => $dataInicioReserva,
              "IDQUALIFICADO"       => $idQualificado
            );
            
            $std = $this->connection->prepare($queryInsertProspect);
            $std->execute($params);
            
            return $cdProspect;
        } else {
            return false;
        }
        
        
    }
    
    public function insertAddressProspect($row, $cdProspect, $nrenderecoparc){
        try {
            $dsendereco       = $row['DSENDERECO'];
            $nrimovelendereco = $row['NRIMOVELENDERECO'];
            $dscompleendereco = $row['DSCOMPLEENDERECO'];
            $nrcependereco    = $row['NRCEPENDERECO'];
            $cdpais           = $row['CDPAIS'];
            $sgestado         = $row['SGESTADO'];
            $cdmunicipio      = $row['CDMUNICIPIO'];
            $cdbairro         = $row['CDBAIRRO'];
            $nmbairroendereco = $row['NMBAIRRO'];
            $nrtelepros       = $row['DSTELEFONECAN'];
            $dsemailpros      = $row['DSEMAILCAN'];
            $cdoperinclusao   = $this->environment->getCdOperador();
            $nrorg            = $row['NRORG'];
            $dtinclusao       = $row['CURRENTDATE'];
            $dsemailpros      = $row['DSEMAILCAN'];
            
            $params = array(
              "NRENDERECOPARC"   => $nrenderecoparc,
              "IDTPENDEPROS"     => 'P',
              "CDPAIS"           => $cdpais,
              "SGESTADO"         => $sgestado,
              "CDMUNICIPIO"      => $cdmunicipio,
              "NRCEPPROS"        => $nrcependereco,
              "DSENDEPROS"       => $dsendereco,
              "NRIMOVELENDERECO" => $nrimovelendereco,
              "DSCOMPENDPRO"     => $dscompleendereco,
              "CDBAIRRO"         => $cdbairro,
              "NMBAIRPROS"       => $nmbairroendereco,
              "NRTELEPROS"       => $nrtelepros,
              "NRTELRESIPRO"     => NULL,
              "NRRAMALPROS"      => NULL,
              "NRFAXPROS"        => NULL,
              "NRTELEXPROS"      => NULL,
              "DSEMAILPROS"      => $dsemailpros,
              "NMCONTPROS"       => NULL,
              "NRRAMALPROS1"     => NULL,
              "NRORG"            => $nrorg,
              "CDPROSPECT"       => $cdProspect,
              "DTINCLUSAO"       => $dtinclusao,
              "CDOPERINCLUSAO"   => $cdoperinclusao,
              "NRORGINCLUSAO"    => $nrorg);
    
            $sql = "INSERT INTO ENDEPROSPECT (NRENDERECOPARC,IDTPENDEPROS,CDPAIS,SGESTADO,CDMUNICIPIO,NRCEPPROS,DSENDEPROS,NRIMOVELENDERECO,DSCOMPENDPRO,CDBAIRRO,NMBAIRPROS,NRTELEPROS,NRTELRESIPRO,NRRAMALPROS,NRFAXPROS,NRTELEXPROS,DSEMAILPROS,NMCONTPROS,NRRAMALPROS1,NRORG,CDPROSPECT,DTINCLUSAO,CDOPERINCLUSAO,NRORGINCLUSAO) 
            VALUES (:NRENDERECOPARC,:IDTPENDEPROS,:CDPAIS,:SGESTADO,:CDMUNICIPIO,:NRCEPPROS,:DSENDEPROS,:NRIMOVELENDERECO,:DSCOMPENDPRO,:CDBAIRRO,:NMBAIRPROS,:NRTELEPROS,:NRTELRESIPRO,:NRRAMALPROS,:NRFAXPROS,:NRTELEXPROS,:DSEMAILPROS,:NMCONTPROS,:NRRAMALPROS1,:NRORG,:CDPROSPECT,:DTINCLUSAO,:CDOPERINCLUSAO,:NRORGINCLUSAO)";
        
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        } catch (\Exception $e) {
            throw $e;
        }
  
    } 
    
    public function disableLead($row){
        
        $queryDesativLead = "UPDATE CRM_CANDIDATO
                                SET IDATIVO = :IDATIVO
                            WHERE NRCANDIDATO = :NRCANDIDATO AND
                                NRORG = :NRORG";
        $params = array (
            "IDATIVO" => "N",
            "NRCANDIDATO" => $row['NRCANDIDATO'],
            "NRORG" => $row['NRORG']
        );
        try
        {
            $std = $this->connection->prepare($queryDesativLead);
            $std->execute($params);
        }catch (\Exception $e)
        {   
            throw $e;
        }
    }
    
    public function procedureMergeCandidate(Row $row, Response $response)
    {
        $row = $row->getRow();

        $nrorg           = $row['NRORG'];
        $cdoperevol      = $row['CDOPEREVOL'];
        $nmprincipalparc = $row['PARCTOSTRING'];
        $nrparcnegocio   = $row['NRPARCNEGOCIO'];
        $retorno         = "";
        $retornoerro     = "";
        $this->procedureFundirParcnegocio->addParam(new Param('P_PARCORIGEM', Param::PARAM_INPUT, $nmprincipalparc));
        $this->procedureFundirParcnegocio->addParam(new Param('P_PARCNEGDESTINO', Param::PARAM_INPUT, $nrparcnegocio));
        $this->procedureFundirParcnegocio->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg));
        $this->procedureFundirParcnegocio->addParam(new Param('P_CDOPERULTATU', Param::PARAM_INPUT, $cdoperevol));
        $this->procedureFundirParcnegocio->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, $retorno));
        $this->procedureFundirParcnegocio->addParam(new Param('P_RETORNOERRO', Param::PARAM_OUTPUT, $retornoerro));

        try{
            $this->procedureFundirParcnegocio->execute();
            $response->addDataSet(new DataSet("DADOS", array("RETORNO" => $retorno, "RETORNOERRO" => $retornoerro)));
        }catch (Exception $e) {
            $response->addDataSet(new DataSet("DADOS", array("RETORNOERRO" => "ERRO AO EXECUTAR A PROCEDURE")));
        }


    }
    
    public function procedureInsertNextTaskClient (Row $row, Response $response) {
        $row = $row->getRow();
        $row["DSFILACONTATO"] = empty($row["DSFILACONTATO"]) ? $row["NMSEQRESULEVOL"] : $row["DSFILACONTATO"];

        $nrseqfilaconta = $this->leadService->prepareInsertLeadEvolutionNextTask($row);
        
        $response->addDataSet(new DataSet("DADOS", array("NRSEQFILACONTA" => $nrseqfilaconta)));
        
        if($row["CDOPEREVOL"] != $row["CDOPERREAL"] && !empty($row["DSEMAILOPER"])) {
        
            $paramsEmail = array(
                "TIPO"                  => "",
                "NMOPERREAL"            => empty($row["NMOPERADORREAL"]) ? '' : $row["NMOPERADORREAL"],
                "OPERADORDOLOGADO"      => empty($row["NMOPERADOR"]) ? '' : $row["NMOPERADOR"],
                "DSTIPO"                => null,
                "DESCRICAO"             => null,
                "NRSEQCONTATO"          => null,
                "DSFILACONTATO"         => empty($row["DSFILACONTATO"]) ? $row["NMSEQRESULEVOL"] : $row["DSFILACONTATO"],
                "NMPARCNEGOCIO"         => empty($row["NMPARCNEGOCIO"]) ? '' : $row["NMPARCNEGOCIO"],
                "DTPLANFILACONTA"       => empty($row["DTPLANFILACONTA"]) ? '' : $row["DTPLANFILACONTA"],
                "DTVENCFILACONTA"       => null,
                "DSCLASSIFICATAREFA"    => null,
                "DSDETEFILACONTA"       => null,
                "DSEMAILOPER"           => empty($row["DSEMAILOPER"]) ? '' : $row["DSEMAILOPER"]
            );
            
            $this->controlerNovaTarefa->enviaEmailTarefa($paramsEmail);
        }
        
    }
    
    public function setTaskEndDate(Row $row){
        
        $row = $row->getRow();
        
        $this->leadService->callForSetTaskEndDate($row);
        
    }
    
    public function deactivateLeadTask(Row $row){
        
        $row = $row->getRow();
        
        $this->leadService->callForDeactivateLeadTask($row);
        
    }
    
    public function insertProspectInterlocutor($row, $cdprospect){
        
        if(isset($row["NMINTERLOCUTOR"]) && !empty($row["NMINTERLOCUTOR"] && $row["NMINTERLOCUTOR"] != '')){
            
            $interlocutorNewde = NovoCodigo::geraCodigo($row['NRORG'], "CRM_INTERLOCUTOR", 1, 10, false, false);
        
            $paramsInsertInterlocutor = array(
                
                "NRSEQINTERLOCUTOR"     => $interlocutorNewde,
                "NMINTERLOCUTOR"        => $row["NMINTERLOCUTOR"],
                "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
                "CDCARGOINTERLOC"       => $row["CDCARGOINTERLOC"],
                "IDRECEBEPRESENTE"      => 'N',
                "EMAILINTERLOC"         => $row["DSEMAILINTERLOCUTOR"],
                "NRTELEFONE1"           => $row["DSTELEFONEINTERLOCUTOR"],
                "NRORG"                 => $row["NRORG"]
                
            );
            
            $this->leadService->callForInsertProspectInterlocutor($paramsInsertInterlocutor);    
            
        }
        
    }
    
    public function insertProspectToLoggedOperatorResale($row) {
        
        $paramsInsert = array(
            "CDPROSPECT"        => $row["CDPROSPECT"],
            "NRREVENDA"         => $row["NRREVENDA"],
            "NRPARCNEGOCIO"     => $row["NRPARCNEGOCIO"],
            "NRORG"             => $row["NRORG"],
            "CDOPERINCLUSAO"    => $this->environment->getCdOperador(),
            "IDATIVO"           => 'S'
        );
        
        $this->leadService->callToInsertProspectToResale($paramsInsert);
        
    }
    
    public function updateLeadPartnership($row){
        $leadPartnershipNumber = self::getLeadPartnershipNumber($row);
        
        if($leadPartnershipNumber){
            
            $leadPartnershipNumber = $leadPartnershipNumber[0]["NRPARCERIA"];
            $insertParams = array(
                "NRPARCERIA"            => $leadPartnershipNumber,
                "NRORG"                 => $row['NRORG'],
                "NRPARCNEGOCIO"         => $row['NRPARCNEGOCIO']
            );
            
            $this->leadService->callToUpdateLeadPartnership($insertParams);
            
        } else {
            
            $row["CDTIPOPARCEIRO"] = "PROSPECT";
            self::createPartnership($row);
            
        }
        
    }
    
    public function getLeadPartnershipNumber($row){
        
        $selectParams = array(
            "NRPARCNEGOCIO"         => $row['NRPARCNEGOCIO'],
            "CDTIPOPARCEIRO"        => "CANDIDATO",
            "NRORG"                 => $row['NRORG']
        );
        
        $partnershipNumber = $this->leadService->callToGetLeadPartnershipNumber($selectParams);
        return $partnershipNumber;
        
    }
    
    public function editLeadResponsibleOperator($row){
        
        if(isset($row['NEWNRPARCPROPRI'])){
            
            $updateParam = array(
                "NRCANDIDATO"       => $row["NRCANDIDATO"],
                "NEWNRPARCPROPRI"   => $row["NEWNRPARCPROPRI"],
                "NRORG"             => $row["NRORG"]
            );
            
            $this->leadService->callToEditLeadResponsibleOperator($updateParam);
            
        }
        
    }
    /*Criado em 21/11/17 por Arthur Herbert
    Método para salvar e updatar enderecoparc*/
    public function saveEnderecoparc($row){
        
        $nrParcnegocio = $row['NRPARCNEGOCIO'];
        $nrOrg = $row['NRORG'];
        $cdoperinclusao = empty($row['CDOPEREVOL']) ?  $row['CDOPERINCLUSAO'] : $row['CDOPEREVOL'];
        $cdPais = empty($row['CDPAIS']) ? '' : $row['CDPAIS'];
        $sgEstado = empty($row['SGESTADO']) ? '' : $row['SGESTADO'];
        $cdMunicipio = empty($row['CDMUNICIPIO']) ? '' : $row['CDMUNICIPIO'];
        $cdBairro = empty($row['CDBAIRRO']) ? '' : $row['CDBAIRRO'];
        $nrCependereco = empty($row['NRCEPENDERECO']) ? '' : $row['NRCEPENDERECO'];
        $dsEndereco = empty($row['DSENDERECO']) ? '' : $row['DSENDERECO'];
        $nrImovelendereco = empty($row['NRIMOVELENDERECO']) ? '' : $row['NRIMOVELENDERECO'];
        $dsCompleendereco = empty($row['DSCOMPLEENDERECO']) ? '' : $row['DSCOMPLEENDERECO'];
        $nrEnderecoParc = $this->leadService->getEnderecoparc($nrParcnegocio, $nrOrg);
        
        if(empty($nrEnderecoParc)){
            $params = array(
              "NRORG"          => $nrOrg,
              "NRPARCNEGOCIO"  => $nrParcnegocio,
              "IDATIVO"        => 'S',
              "CDOPERINCLUSAO" => $cdoperinclusao,
              "CDPAIS"         => $cdPais,
              "SGESTADO"       => $sgEstado,
              "CDMUNICIPIO"    => $cdMunicipio,
              "CDBAIRRO"       => $cdBairro,
              "NRENDERECOPARC" => $nrEnderecoParc,
              "NRCEPENDERECO"  => $nrCependereco,
              "DSENDERECO"     => $dsEndereco,
              "NRIMOVELENDERECO" => $nrImovelendereco,
              "DSCOMPLEENDERECO"  => $dsCompleendereco);
            $nrEnderecoParc = NovoCodigo::geraCodigo($row["NRORG"], "ENDERECOPARC", 1, 10, false, false);
            $params['NRENDERECOPARC'] = $nrEnderecoParc;
            $this->leadService->insertEnderecoParc($params);
        }else{
             $params = array(
              "CDPAIS"         => $cdPais,
              "SGESTADO"       => $sgEstado,
              "CDMUNICIPIO"    => $cdMunicipio,
              "CDBAIRRO"       => $cdBairro,
              "NRPARCNEGOCIO"  => $nrParcnegocio,
              "NRENDERECOPARC" => $nrEnderecoParc,
              "NRORG"          => $nrOrg,
              "NRCEPENDERECO"  => $nrCependereco,
              "DSENDERECO"     => $dsEndereco,
              "NRIMOVELENDERECO" => $nrImovelendereco,
              "DSCOMPLEENDERECO"  => $dsCompleendereco);
            $this->leadService->updateEnderecoParc($params);
        }
    }
    
    public function saveComunicaparc($row, $response){
        if(!empty($row['DSEMAILCAN'])){
            $comunicaparc = $this->leadService->getComunicaparc($row['DSEMAILCAN'], array('05'), $row['NRPARCNEGOCIO'], $this->environment->getNrOrg());
            if(empty($comunicaparc)){
                $params = array(
                        "CDCOMUNICAPARC" => $row['DSEMAILCAN'],
                        "IDATIVO"        => 'S',
                        "NRCOMUNICAPARC" => NovoCodigo::geraCodigo($this->environment->getNrOrg(), "COMUNICAPARC", 1, 10, false, true),
                        "NRPARCNEGOCIO"  => $row['NRPARCNEGOCIO'],
                        "CDFORMACOMU"    => '05',
                        "NRORG"          => $this->environment->getNrOrg()
                    );
                if(!$this->leadService->insertComunicaparc($params)){
                    $response->addDataSet(new DataSet("ERRO", array("MESSAGE" => 'Não foi possível salvar o email.'), 10));   
                }
                
            }
        }
        if(!empty($row['DSTELEFONECAN'])){
            $comunicaparc = $this->leadService->getComunicaparc($row['DSTELEFONECAN'], array('01', '02', '03', '04'), $row['NRPARCNEGOCIO'], $this->environment->getNrOrg());
            if(empty($comunicaparc)){
                $comunicaparc = $this->leadService->getTelPrinc($row['NRPARCNEGOCIO'], $this->environment->getNrOrg());
                if(empty($comunicaparc)){
                    $params = array(
                            "CDCOMUNICAPARC" => $row['DSTELEFONECAN'],
                            "IDATIVO"        => 'S',
                            "NRCOMUNICAPARC" => NovoCodigo::geraCodigo($this->environment->getNrOrg(), "COMUNICAPARC", 1, 10, false, true),
                            "NRPARCNEGOCIO"  => $row['NRPARCNEGOCIO'],
                            "CDFORMACOMU"    => '01',
                            "NRORG"          => $this->environment->getNrOrg()
                        );  
                    if(!$this->leadService->insertComunicaparc($params)){
                        $response->addDataSet(new DataSet("ERRO", array("MESSAGE" => 'Não foi possível salvar o telefone.'), 10));   
                    }
                } else {
                    if(!$this->leadService->updateTelPrinc($row['DSTELEFONECAN'], $row['NRPARCNEGOCIO'], $this->environment->getNrOrg())){
                        $response->addDataSet(new DataSet("ERRO", array("MESSAGE" => 'Não foi possível salvar o telefone.'), 10));   
                    }
                    
                }
            }
        }
    }
    public function saveDtnascifundparc($row){
        $params = array(
            "NRPARCNEGOCIO"   => $row['NRPARCNEGOCIO'],
            "DTNASCIFUNDPARC" => !empty($row['DTNASCIFUNDPARC']) ? $row['DTNASCIFUNDPARC'] : "",
            "NRORG"           => $this->environment->getNrOrg()
        );  
        $this->leadService->saveDtnascifundparc($params);   
    }
    
    public function createPartnershipNotExistent($row, $tipo = 'PROSPECT'){
        $nrParcnegocio = $row['NRPARCNEGOCIO'];
        $nrOrg = $row['NRORG'];
        
        // verifica se não foi adicionado uma parceria para o tipo do parceiro no meio tempo de execução da rotina
        $existente = $this->leadService->verifyExistentPartnership($nrParcnegocio, $tipo, $nrOrg); 
        
        if($existente) {
            $nrparceiro = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, true);
            $cdoperinclusao = isset($row['CDOPEREVOL']) ? $row['CDOPEREVOL'] : $this->environment->getCdOperador();
            
            $params = array(
              "NRPARCERIA"     => $nrparceiro,
              "NRORG"          => $nrOrg,
              "NRPARCNEGOCIO"  => $nrParcnegocio,
              "CDTIPOPARCEIRO" => $tipo,
              "IDATIVO"        => 'S',
              "CDOPERINCLUSAO" => $cdoperinclusao,
              "NRORGINCLUSAO"  => $nrOrg);
              
            $queryInsertParceria = "INSERT INTO PARCERIA(NRPARCERIA, NRORG, NRPARCNEGOCIO, CDTIPOPARCEIRO, IDATIVO, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
            VALUES(:NRPARCERIA, :NRORG, :NRPARCNEGOCIO, :CDTIPOPARCEIRO, :IDATIVO, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";
    
            $std = $this->connection->prepare($queryInsertParceria);
            $std->execute($params);
        }
    }
    
    public function searchCountry(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $paramsRequest = $request->getParameters()['paramsToSearch'];
            $params = array(
                "P_NRORG"        => $this->environment->getNrOrgtrab(),
                "P_NRORGPADRAO"  => 0,
                "P_CDPAIS"       => $paramsRequest[0],
                "P_SGESTADO"     => $paramsRequest[1],
                "P_CDMUNICIPIO"  => $paramsRequest[2],
                "P_CDBAIRRO"     => $paramsRequest[3]
            );
              
            $query = "
                SELECT 
                  PAIS.NMPAIS,
                  PAIS.CDPAIS,
                  ESTADO.NMESTADO,
                  ESTADO.SGESTADO,
                  MUNICIPIO.NMMUNICIPIO,
                  MUNICIPIO.CDMUNICIPIO,
                  BAIRRO.CDBAIRRO,
                  BAIRRO.NMBAIRRO
                FROM 
                  PAIS 
                  LEFT JOIN ESTADO
                    ON ESTADO.CDPAIS = PAIS.CDPAIS
                      AND (ESTADO.NRORG = PAIS.NRORG
                            OR ESTADO.NRORG = :P_NRORGPADRAO)
                  LEFT JOIN MUNICIPIO
                    ON MUNICIPIO.CDPAIS = PAIS.CDPAIS
                      AND MUNICIPIO.SGESTADO = ESTADO.SGESTADO
                      AND (MUNICIPIO.NRORG = PAIS.NRORG
                            OR MUNICIPIO.NRORG = :P_NRORGPADRAO)
                  LEFT JOIN BAIRRO
                    ON BAIRRO.CDPAIS = PAIS.CDPAIS
                      AND BAIRRO.SGESTADO = ESTADO.SGESTADO
                      AND BAIRRO.CDMUNICIPIO = MUNICIPIO.CDMUNICIPIO
                      AND (BAIRRO.NRORG = PAIS.NRORG
                            OR BAIRRO.NRORG = :P_NRORGPADRAO)
                WHERE 
                  PAIS.CDPAIS = :P_CDPAIS
                  AND (PAIS.NRORG = :P_NRORG
                        OR PAIS.NRORG = :P_NRORGPADRAO)
                  AND (ESTADO.SGESTADO = :P_SGESTADO
                        OR :P_SGESTADO IS NULL)
                  AND (MUNICIPIO.CDMUNICIPIO = :P_CDMUNICIPIO
                        OR :P_CDMUNICIPIO IS NULL)
                  AND (BAIRRO.CDBAIRRO = :P_CDBAIRRO
                        OR :P_CDBAIRRO IS NULL)
                   
            ";
    
            $std = $this->connection->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            $response->addDataSet(new DataSet("dados", $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function updateResponsavel($row) {
        try {
            $nrparcoper = NULL;
            $cdoper = NULL;
            
            if(!empty($row['NRPARCPROPRI'])) {
                $operador = $this->getOperador($row['NRORG'], $row['NRPARCPROPRI']);
                if(!empty($operador)) {
                    $nrparcoper = $operador['NRPARCNEGOCIO'];
                    $cdoper = $operador['CDOPERADOR'];
                    if(empty($nrparcoper)) {
                        $nrparcoper = $operador[0]['NRPARCNEGOCIO'];
                        $cdoper = $operador[0]['CDOPERADOR'];
                    }
                }
            }
            
            $updateProspect = "UPDATE CRM_STATUSPARC
                               SET NRPARCRESP = :NRPARCRESP,
                                   CDOPERRESP = :CDOPERRESP
                               WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO 
                                     AND NRORG = :NRORG
                                     AND CRM_NRORGTRAB = :CRM_NRORGTRAB";
            
            $params = array (
                "NRORG"             => $row['NRORG'],
                "CRM_NRORGTRAB"     => $row['CRM_NRORGTRAB'],
                "NRPARCNEGOCIO"     => $row['NRPARCNEGOCIO'],
                "NRPARCRESP"        => $nrparcoper,
                "CDOPERRESP"        => $cdoper
            );
            
            $std = $this->connection->prepare($updateProspect);
            $std->execute($params);
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
   
    public function setRelationshipPartnerStatus(DTO\Request $request, DTO\Response $response) {
        try {
            
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $prospect = $this->verifyProspect($row);
                
                $lead = $this->getLead($row['NRORG'], $row['NRPARCNEGOCIO']);

                if(!empty($lead)) {
                    $operador = $this->getOperador($row['NRORG'], $lead['NRPARCPROPRI']);
                    $address = $this->leadService->returnParcAddress($row['NRPARCNEGOCIO'], $row['NRORG']);
                    
                    $lead['CDOPERRESPONSAVEL'] = !empty($operador['CDOPERADOR']) ? $operador['CDOPERADOR'] : null;
                    $lead['NRPARCRESPROSP'] = $lead['NRPARCPROPRI'];
                    $lead['CRM_NRORGTRAB'] = $row['CRM_NRORGTRAB'];
                    
                    $lead['LATITUDE'] = $address['DSLATITUDE'];
                    $lead['LONGITUDE'] = $address['DSLONGITUDE'];
                    $lead['CURRENTDATE'] = $row['DTINCLUSAO'];
                    $lead['CDPAIS'] = $address['CDPAIS'];
                    $lead['NMPAIS'] = $address['NMPAIS'];
                    $lead['SGESTADO'] = $address['SGESTADO'];
                    $lead['NMESTADO'] = $address['NMESTADO'];
                    $lead['CDMUNICIPIO'] = $address['CDMUNICIPIO'];
                    $lead['NMMUNICIPIO'] = $address['NMMUNICIPIO'];
                    $lead['CDBAIRRO'] = $address['CDBAIRRO'];
                    $lead['NMBAIRRO'] = $address['NMBAIRRO'];
                    $lead['NRIMOVELENDERECO'] = $address['NRIMOVELENDERECO'];
                    $lead['NRCEPENDERECO'] = $address['NRCEPENDERECO'];
                    $lead['DSCOMPLEENDERECO'] = $address['DSCOMPLEENDERECO'];
                    $lead['DSENDERECO'] = $address['DSENDERECO'];
                }
                
                if(empty($prospect)) {
                    $return = self::updatePartnerAndReturnAddress($lead);
                    $nrenderecoparc = $return['endereco'];
                    $prospect = $return['prospect'];
                    if(empty($prospect)) {
                        $lead['CDPROSPECT'] = self::insertProspectAndReturnCDprospect($lead);
                    } else {
                        $lead['CDPROSPECT'] = $prospect[0]['CDPROSPECT'];
                    }
                    
                    if(!empty($lead['CDPROSPECT'])) {
                        if(empty($prospect)) {
                            self::insertAddressProspect($lead, $lead['CDPROSPECT'], $nrenderecoparc);
                            self::disableLead($lead);
                            self::createPartnershipNotExistent($lead);
                        }
                        self::editLeadResponsibleOperator($lead);
                        
                        if(isset($lead['NRREVENDA'])){
                            self::insertProspectToLoggedOperatorResale($lead);
                        }
                        
                        $this->saveComunicaparc($lead, $response);
                        $this->saveDtnascifundparc($lead);
                    }
                } else {
                    $this->agrupamentoParceiroService->setRelationshipPartnerStatus($row, 'PROSPECT');
                }
                
                $paramsToEvol = array(
                    "DTHREVOLCONT"      => $row['DTINCLUSAO'],
                    "CDOPEREVOL"        => $row['CDOPEREVOL'],
                    "NRORG"             => $row['NRORG'],
                    "CRM_NRORGTRAB"     => $row['CRM_NRORGTRAB']
                );
                
                $this->agrupamentoParceiroService->agrupamentoEvolSave($paramsToEvol);
                
            }
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
    
    public function verifyProspect($row) {
        try {
            $sql = "SELECT 
                      * 
                    FROM 
                      PROSPECT 
                    WHERE
                      NRPARCNEGOCIO = :NRPARCNEGOCIO
                      AND NRORG = :NRORG";
            
            $params = array (
                "NRORG"             => $row['NRORG'],
                "NRPARCNEGOCIO"     => $row['NRPARCNEGOCIO']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
    
    public function getLead($nrorg, $nrparcnegocio) {
        try {
            $sql = "SELECT 
                      * 
                    FROM 
                      CRM_CANDIDATO 
                    WHERE
                      NRPARCNEGOCIO = :NRPARCNEGOCIO
                      AND NRORG = :NRORG
                      AND ROWNUM = 1";
            
            $params = array (
                "NRORG"             => $nrorg,
                "NRPARCNEGOCIO"     => $nrparcnegocio
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            $lead = $std->fetchAll();
            
            if(!empty($lead)) {
                return $lead[0];
            } else {
                return $lead;
            }
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
    
    public function getOperador($nrorg, $nrparcnegocio) {
        try {
            $sql = "SELECT 
                      CDOPERADOR, 
                      NRPARCNEGOCIO 
                    FROM 
                      OPERADOR 
                    WHERE 
                      NRPARCNEGOCIO = :NRPARCNEGOCIO 
                      AND NRORG = :NRORG 
                      AND ROWNUM = 1";
            
           $paramsOper = array(
               "NRPARCNEGOCIO" => $nrparcnegocio,
               "NRORG" => $nrorg
           );
            
           $std = $this->connection->prepare($sql);
           $std->execute($paramsOper);
           $operador = $std->fetchAll();
           
           if(!empty($operador)) {
               return array('NRPARCNEGOCIO' => $operador[0]['NRPARCNEGOCIO'], 'CDOPERADOR' => $operador[0]['CDOPERADOR']);
           } else {
               return array();
           }
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
    
}
